import os
from pathlib import Path
from typing import List

import aiounittest
from dotenv import load_dotenv
from werk24.models.ask import W24Ask, W24AskPageThumbnail
from werk24.techread_client import (W24TechreadArchitecture,
                                    W24TechreadArchitectureStatus,
                                    W24TechreadClient, W24TechreadMessageType)


class TechreadIntegrationTest(aiounittest.AsyncTestCase):

    """ List of the environment variables that
    are required by the application
    """
    envs_required = [
        "W24TECHREAD_SERVER_HTTPS",
        "W24TECHREAD_SERVER_WSS",
        "W24TECHREAD_VERSION",
        "W24TECHREAD_AUTH_REGION",
        "W24TECHREAD_AUTH_IDENTITY_POOL_ID",
        "W24TECHREAD_AUTH_CLIENT_ID",
        "W24TECHREAD_AUTH_CLIENT_SECRET",
        "W24TECHREAD_AUTH_USERNAME",
        "W24TECHREAD_AUTH_PASSWORD"]

    def setUp(self) -> None:
        self._load_env()
        self.client = W24TechreadClient.make_from_env()
        assert isinstance(self.client, W24TechreadClient)

        # set the drawing path
        base_path = Path(os.path.dirname(os.path.realpath(__file__)))
        self.drawing_path = base_path / "fixtures" / "techdraw.png"

    def _load_env(self)-> None:
        """ load the environment variables and test that they
        are available
        """
        load_dotenv(".werk24")
        for cur_env in self.envs_required:
            self.assertIsNotNone(os.environ.get(cur_env))

    def _get_drawing_bytes(self)-> bytes:
        with open(self.drawing_path, "rb") as drawing_handle:
            return drawing_handle.read()

    @staticmethod
    def _make_asks_list()-> List[W24Ask]:
        return [
            W24AskPageThumbnail()
        ]

    async def test_techread(self) -> None:

        # start a session
        async with self.client as session:

            # check whether we have obtained a token
            self.assertIsNotNone(session._auth_client.token)  # noqa

            # get the architecture status
            architecture_status = await session.get_architecture_status(
                W24TechreadArchitecture.GPU_V1)

            # check whether that value is valid
            self.assertIsInstance(
                architecture_status,
                W24TechreadArchitectureStatus)

            # now get the file
            drawing_bytes = self._get_drawing_bytes()

            asks_list = self._make_asks_list()

            # and make the request
            is_first_message = True
            async for message in session.read_drawing(
                    drawing_bytes, asks_list):

                # the first message needs to be a TECHREAD_STARTED
                if is_first_message:
                    self.assertEqual(
                        message.message_type,
                        W24TechreadMessageType.PROGRESS)
                    is_first_message = False

                # if the message tells us about an error, we nee to raise it
                self.assertNotEqual(
                    message.message_type,
                    W24TechreadMessageType.ERROR)



""" Command Line Interface for W24 Techread
"""
import argparse
import io
import logging
import sys
from typing import List

from dotenv import load_dotenv
from werk24.exceptions import RequestTooLargeException
from werk24.models.ask import (W24AskCanvasThumbnail, W24AskPageThumbnail,
                               W24AskSectionalThumbnail, W24AskSheetThumbnail,
                               W24AskVariantGDTs, W24AskVariantMeasures)
from werk24.models.techread import (W24TechreadMessageSubtypeError,
                                    W24TechreadMessageSubtypeProgress,
                                    W24TechreadMessageType)
from werk24.techread_client import Hook, LicenseError, W24TechreadClient

# load the environment variables
load_dotenv(".werk24")

# set the log level to info for the test setting
# We recommend using logging.WARNING for production
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s.%(msecs)03d %(levelname)s %(module)s - %(funcName)s: %(message)s',  # noqa
    datefmt='%Y-%m-%d %H:%M:%S')

# get the local logger
logger = logging.getLogger(__name__)  # pylint:disable = invalid-name


def _get_drawing(
        file_path: str
) -> bytes:
    """ Obtain the bytes content of the test drawing
    """
    # get the content
    with open(file_path, "rb") as filehandle:
        return filehandle.read()


def _debug_show_image(
        log_text: str,
        image_bytes: bytes
) -> None:

    # Import the PIL at runtime to make sure that
    # you can still use the client without the need
    # to install Pillow
    try:
        from PIL import Image  # pylint: disable=import-outside-toplevel
    except ImportError:
        logger.warning(
            "Viewing image-like responses, requires the installation"
            " of the PIL package: pip install pillow. Image skipped.")
        return

    # otherwise print the text
    logger.info(log_text)

    # and show the image
    image = Image.open(io.BytesIO(image_bytes))
    image.show(title=log_text)


async def main(
    args: argparse.Namespace
) -> None:
    """ Run the Techread command with the CLI arguments

    Arguments:
        args {argparse.Namespace} -- CLI args generated by
            argparse
    """

    # make the hooks from the arguments
    hooks = _make_hooks_from_args(args)

    # make the client. This will automatically
    # fetch the authentication information
    # from the environment variables. We will
    # provide you with separate .env files for
    # the development and production environments
    try:
        client = W24TechreadClient.make_from_env()

    # If a license error occured, let the user know.
    except LicenseError as exception:
        print(f"LICENSE ERROR: {exception}")
        sys.exit()

    async with client as session:

        # get the drawing
        drawing_bytes = _get_drawing(args.input_file)

        # and make the request
        try:
            await session.read_drawing_with_hooks(
                drawing_bytes,
                hooks)

        except RequestTooLargeException:
            message = "Request was too large to be processed. " \
                + "Please check the documentation for current limits."
            logger.error(message)


def _make_hooks_from_args(
        args: argparse.Namespace
) -> List[Hook]:

    # tell the api what asks you are interested in,
    # and define what to do when you receive the result
    hooks = []

    # add the hook to get the information that the techread
    # process was started

    # add the hook for the page thumbnail
    if args.ask_page_thumbnail:
        hooks += [Hook(
            ask=W24AskPageThumbnail(),
            function=lambda msg: _debug_show_image(
                "Page Thumbnail received",
                msg.payload_bytes))]

    # add the hook for the sheet thumbnail
    if args.ask_sheet_thumbnail:
        hooks += [Hook(
            ask=W24AskSheetThumbnail(),
            function=lambda msg: _debug_show_image(
                "Sheet Thumbnail received",
                msg.payload_bytes))]

    # add the hook for the sheet thumbnail
    if args.ask_canvas_thumbnail:
        hooks += [Hook(
            ask=W24AskCanvasThumbnail(),
            function=lambda msg: _debug_show_image(
                "Canvas Thumbnail received",
                msg.payload_bytes))]

    # add the hook for the drawing thumbnail
    if args.ask_sectional_thumbnail:
        hooks += [Hook(
            ask=W24AskSectionalThumbnail(),
            function=lambda msg: _debug_show_image(
                "Drawing thumbnail received",
                msg.payload_bytes))]

    # add the hook for the variant measures
    if args.ask_variant_measures:
        hooks += [Hook(
            ask=W24AskVariantMeasures(),
            function=lambda msg: print(
                "Received Ask Variant Measures\n",
                msg.payload_dict
            ))]

        # add the hook for the variant measures
    if args.ask_variant_gdts:
        hooks += [Hook(
            ask=W24AskVariantGDTs(),
            function=lambda msg: print(
                "Received Ask Variant GDTs\n",
                msg.payload_dict
            ))]

    # add a general hook to deal with internal errors
    hook_error_internal = Hook(
        message_type=W24TechreadMessageType.ERROR,
        message_subtype=W24TechreadMessageSubtypeError.INTERNAL,
        function=lambda msg:
        logger.error("Internal Error %s", msg.payload_dict))
    hooks.append(hook_error_internal)

    # add a gneral hook to deal with PROGRESS messages
    hook_progress_started = Hook(
        message_type=W24TechreadMessageType.PROGRESS,
        message_subtype=W24TechreadMessageSubtypeProgress.STARTED,
        function=lambda msg: logger.info("Techread process started"))
    hooks.append(hook_progress_started)

    return hooks



""" Defintion of all W24Ask types that are understood by the Werk24 API.
"""
from enum import Enum
from typing import List

from pydantic import UUID4, BaseModel

from .gdt import W24GDT
from .measure import W24Measure


class W24AskType(str, Enum):
    """ List of all Ask Type supported by the current
    API version.

    """
    PAGE_THUMBNAIL = "PAGE_THUMBNAIL"
    """ Thumbnail of the overall page - rotated and with
    surrounding white space removed """

    SHEET_THUMBNAIL = "SHEET_THUMBNAIL"
    """ Thumbnail of the sheet (i.e., the part of the
    page that is described by the surrounding frame) """

    CANVAS_THUMBNAIL = "CANVAS_THUMBNAIL"
    """ Thumbnail of the canvas (i.e., the part of the
    sheet that contains the geometry)
    """

    SECTIONAL_THUMBNAIL = "SECTIONAL_THUMBNAIL"
    """ Thumbnail of a sectional on the canvas.
    Here the sectional describes both cuts and perspectives
    """

    VARIANT_MEASURES = "VARIANT_MEASURES"
    """ List of Measures that were found on the Sectionals
    associated with the variant
    """

    VARIANT_GDTS = "VARIANT_GDTS"
    """ List of Geometric Dimensions and Tolerations detected
    on the Sectionals associated with the variant
    """

    TRAIN = "TRAIN"
    """ Supplying the request for training only without
    expecting a response.
    """


class W24Ask(BaseModel):
    """ Base model from wich all Asks inherit
    """
    ask_type: W24AskType


class W24AskThumbnailFileFormat(str, Enum):
    """ List of supported File Formats in which
    the Thumbnail can be supplied.

    NOTE: At this stage, the API only supports JPEG.
    This prepares the support for additional format
    (e.g. TIFF)
    """
    JPEG = "JPEG"


class W24AskThumbnail(W24Ask):
    """ Base model for features that request a thumbnail.

    NOTE: At this stage, the API will return a high-resolution
    image. Future releases will allow the defintion of the
    maximal dimensions.
    """
    file_format: W24AskThumbnailFileFormat = W24AskThumbnailFileFormat.JPEG


class W24AskPageThumbnail(W24AskThumbnail):
    """ Requests a thumbnail for each page in the document;
    rotated, and with the surrounding white-space removed.

    NOTE: when you supply a white-on-black document, the thumbnail
    will be black-on-white.
    """
    ask_type = W24AskType.PAGE_THUMBNAIL


class W24AskSheetThumbnail(W24AskThumbnail):
    """ Requests a thumbnail of each sheet on each page in
    the document. The sheet will only contain the pixels within
    the main frame that surrounds the canvas and header fields.
    """
    ask_type = W24AskType.SHEET_THUMBNAIL


class W24AskCanvasThumbnail(W24AskThumbnail):
    """ Requests a thumbnail of each canvas in each sheet.
    The canvas describes the "drawing area" of the sheet.
    """
    ask_type = W24AskType.CANVAS_THUMBNAIL


class W24AskSectionalThumbnail(W24AskThumbnail):
    """ The W24AskPlaneThumbnail requests a thumbnail
    of each sectional on each sheet in the document.

    NOTE: we have chosen the term sectional to describe
    both cuts and perspectives
    """
    ask_type = W24AskType.SECTIONAL_THUMBNAIL


class W24AskVariantMeasures(W24Ask):
    """ With this Ask you are requesting the complete
    list of all measures that were detected for the
    variant
    """
    ask_type = W24AskType.VARIANT_MEASURES


class W24AskVariantMeasuresResponse(BaseModel):
    """ Response object corresponding to the
    W24AskVariantMeasures ask.

    NOTE: Be aware that requesting the measures will
    yield one responds for each variant and sectional
    """
    variant_id: UUID4
    sectional_id: UUID4
    measures: List[W24Measure]


class W24AskVariantGDTs(W24Ask):
    """ This Ask requests the complete
    list of all Geometric Dimensions and Tolerations
    that were detected on the variant.
    """
    ask_type = W24AskType.VARIANT_GDTS


class W24AskVariantGDTsResponse(BaseModel):
    """ Response object corresponding ot hte
    W24AskVariantGDTs.

    NOTE: Be aware that requesting the measures will
    yield one responds for each variant and sectional
    """
    variant_id: UUID4
    sectional_id: UUID4
    gdts: List[W24GDT]


class W24AskTrain(BaseModel):
    """ If you submit this Ask, we will use your request
    to train and improve our models.
    It does not trigger a response
    """
    ask_type = W24AskType.TRAIN



from PyQt5 import Qsci


class W24GuiJsonEditor(Qsci.QsciScintilla):
    """ Small JSON Editor with highlighter functionality
    to make it a bit easier for the user to read the
    JSON responses that come from the API
    """

    def __init__(self, parent: int = None) -> None:
        super().__init__(parent)
        self.setLexer(Qsci.QsciLexerJSON(self))



""" Just your basic unit defintions

Author: Jochen Mattes - Werk24
"""
from enum import Enum


class W24UnitLength(str, Enum):
    """ Enum of the supported length units
    """
    MILLIMETER = "MILLIMETER"
    INCH = "INCH"



import io
from typing import Any, Dict, List, Tuple

from PIL import Image, ImageDraw, ImageFont
from werk24.models.gdt import W24GDT
from werk24.models.measure import W24Measure

try:
    font = ImageFont.truetype('arial.ttf', 30)
except IOError:
    font = ImageFont.load_default()


def _load_image(image_bytes: bytes) -> Tuple[Image.Image, ImageDraw.ImageDraw]:
    img = Image.open(io.BytesIO(image_bytes)).convert('RGB')
    draw = ImageDraw.Draw(img)
    return img, draw


def illustrate_sectional_gdts(
        sectional_bytes: bytes,
        gdts: List[W24GDT]
) -> bytes:

    # open the image from the bytes buffer
    # and dervice the height and width
    img, draw = _load_image(sectional_bytes)
    width, height = img.size

    for cur_gdt_obj in gdts:
        cur_gdt = W24GDT.parse_obj(cur_gdt_obj)

        # make a shorter handle for the polygon, and
        # make sure that the polygon does exist and
        # has at least 4 points
        poly = cur_gdt.bounding_polygon
        if not poly or len(poly) < 4:
            continue

        # then draw the polygon onto the image
        for pt1, pt2 in zip(poly, poly[1:] + [poly[0]]):
            draw.line(
                [pt1[0] * width, pt1[1] * height,
                 pt2[0] * width, pt2[1] * height],
                fill=(0, 255, 0),
                width=4)

        # now... we want to have a measure label
        # that is on the center coordinate of the
        # measure
        left = min([float("inf")] + [pt[0] for pt in poly])
        top = min([float("inf")] + [pt[1] for pt in poly])
        right = max([0.0] + [pt[0] for pt in poly])
        bottom = max([0.0] + [pt[1] for pt in poly])
        x_center = (left + right) / 2 * width
        y_center = (top + bottom) / 2 * height
        if left == float("inf") or top == float("inf"):
            continue

        # cur_measure.label.blurb
        draw.text(
            [x_center, y_center],
            cur_gdt.frame.blurb,
            fill=(0, 200, 0),
            font=font)

    return img.tobytes("jpeg", "RGB")


def illustrate_sectional_measures(
        sectional_bytes: bytes,
        measures: List[Dict[str, Any]]
) -> bytes:

    # open the image from the bytes buffer
    # and dervice the height and width
    img, draw = _load_image(sectional_bytes)
    width, height = img.size

    for cur_measure_obj in measures:
        # parse the measure as local W24Measure
        cur_measure = W24Measure.parse_obj(cur_measure_obj)
        cur_line = cur_measure.line

        # draw a line that corresponds to the
        # measure line
        draw.line(
            [cur_line[0][0] * width,
             cur_line[0][1] * height,
             cur_line[1][0] * width,
             cur_line[1][1] * height],
            fill=(0, 255, 0),
            width=4)

        # now... we want to have a measure label
        # that is on the center coordinate of the
        # measure
        x_center = (cur_line[0][0] + cur_line[1][0]) / 2 * width
        y_center = (cur_line[0][1] + cur_line[1][1]) / 2 * height
        # cur_measure.label.blurb
        draw.text(
            [x_center, y_center],
            cur_measure.label.blurb,
            fill=(0, 200, 0),
            font=font)

    return img.tobytes("jpeg", "RGB")



""" Defintions of all objects required to communicate with
the W24 Techread API.
"""
from enum import Enum
from typing import Dict, List, Optional, Union

from pydantic import UUID4, BaseModel, HttpUrl, Json

from .ask import W24Ask, W24AskType


class W24TechreadAction(str, Enum):
    """ List of supported actions by the Techread API
    """
    INITIALIZE = "INITIALIZE"
    READ = "READ"


class W24TechreadCommand(BaseModel):
    """ Command that is sent from the client to the Server
    """
    action: W24TechreadAction
    message: Json


class W24TechreadMessageType(str, Enum):
    """ Message Type of the message that is sent
    from the server to the client in response to
    a request.
    """
    ASK = "ASK"
    ERROR = "ERROR"
    PROGRESS = "PROGRESS"
    REJECTION = "REJECTION"


class W24TechreadMessageSubtypeError(str, Enum):
    """ Message Subtype for the MessageType: ERROR
    """
    INTERNAL = "INTERNAL"


class W24TechreadMessageSubtypeRejection(str, Enum):
    """ Message Subtype for the MessageType: REJECTION
    """
    COMPLEXITY_EXCEEDED = "COMPLEXITY_EXCEEDED"
    PAPER_SIZE_LIMIT_EXCEEDED = "PAPER_SIZE_LIMIT_EXCEEDED"


class W24TechreadMessageSubtypeProgress(str, Enum):
    """ Message Subtype for the MessageType: PROGRESS
    """
    INITIALIZATION_SUCCESS = "INITIALIZATION_SUCCESS"
    COMPLETED = "COMPLETED"
    STARTED = "STARTED"


W24TechreadMessageSubtypeAsk = W24AskType
""" The MessageType: ASK will return the subtypes
defined in W24AskTypes
"""

W24TechreadMessageSubtype = Union[
    W24TechreadMessageSubtypeError,
    W24TechreadMessageSubtypeProgress,
    W24TechreadMessageSubtypeAsk]
""" Shorthand to summorize all the supported
MessageTypes
"""


class W24TechreadMessage(BaseModel):
    """ Message format for messages that are send
    from the server to the client.
    """
    request_id: UUID4
    """ unique UUID4 that is generated by the
    server to identify the request
    """

    message_type: W24TechreadMessageType
    """ Main Message Type (see W24TechreadMessageType)
    """

    message_subtype: W24TechreadMessageSubtype
    """ Message SubType (see W24TechreadMessageSubtype)
    """

    payload_dict: Optional[Dict] = None
    """ Payload dictionary containing the response
    as dict. The MessageType/Subtype will tell the
    interpreter how to turn the payload back into
    the corresponding object
    """

    payload_url: Optional[HttpUrl] = None
    """ For binary data, the API will return a download
    url which carries the data. This allows us to transfer
    larger images etc.
    """

    payload_bytes: Optional[bytes] = None
    """ Binary reference of the payload. This will only
    become available after the client has downloaded the
    payload_url.
    """


class W24TechreadRequest(BaseModel):
    """ Definition of a W24DrawingReadRequest containing
    all the asks (i.e., things you want to learn about
    the technical drawing).

    """
    asks: List[W24Ask] = []
    """ List of asks """

    development_key: Optional[str] = None
    """ The development_key is used for internal purposes.
    It wil give you access to pre-release versions of our software.
    You will only understand the details if you
    """




""" Defintion of all the W24MeasureWarning class its support structures


Author: Jochen Mattes - Werk24
"""
from enum import Enum
from pydantic import BaseModel


class W24MeasureWarningType(str, Enum):
    """ List of all warnings that can be associated with
    the returned measures.
    """

    UNCONVENTIONAL_TOLERANCE_ORDER = "UNCONVENTIONAL_TOLERANCE_ORDER"
    """ The UNCONVENTIONAL_TOLERANCE_ORDER warning is raised
    when the first-mentioned tolerance is lower than the second-mentioned.

    EXAMPLE: 3 -0.1/+0.1 (rather than 3 +0.1/-0.1)
    """

    UNPROPORTIONAL = "UNPROPORTIONAL"
    """ The UNPROPORTIONAL warnings is set when the size indicated
    on the MeasureLabel is unproportional to the distance between
    the Measure's end-points.
    """


class W24MeasureWarning(BaseModel):
    """ Warnings are issued when something about the label
    or measure is not conforming with convention
    """
    warning_type: W24MeasureWarningType


class W24MeasureWarningUnconvetionalToleranceOrder(W24MeasureWarning):
    """ The UNCONVENTIONAL_TOLERANCE_ORDER warning is raised
    when the first-mentioned tolerance is lower than the second-mentioned.

    EXAMPLE: 3 -0.1/+0.1 (rather than 3 +0.1/-0.1)
    """
    warning_type = W24MeasureWarningType.UNCONVENTIONAL_TOLERANCE_ORDER


class W24MeasureWarningUnproportional(W24MeasureWarning):
    """ The UNPROPORTIONAL warnings is set when the size indicated
    on the MeasureLabel is unproportional to the distance between
    the Measure's end-points.

    NOTE: Three things can cause this:
    1. The Measure is truely unpropotional and carries an associated
        indicator.

    2. The Measure is unporportional, but not indicated as such.
        Typically seen on drawings that "were modified on the fly"

    3. The size of the MeasureLabel was misread by the Werk24
        backend.

    Due to the lack of training data, we can currently not
    differentiate between case 1 and case 2. Once the training
    data suffices, this warning will be extended by boolean
    field 'indicated'.

    In addition, it is theoretically possible to estimate
    the likelihood of case 2 and case 3. If this becomes a
    true concern in your application, please reach out to us
    """
    warning_type = W24MeasureWarningType.UNPROPORTIONAL



from typing import Optional

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon, QImage, QPixmap
from PyQt5.QtWidgets import QFrame, QGridLayout, QLabel, QVBoxLayout
from werk24.gui.json_editor import W24GuiJsonEditor
from werk24.gui.style import ft_headline


class W24GuiEventFeed(QVBoxLayout):
    """ API Event feed listing all the
    responses that we obtained from the
    API
    """

    def add_welcome(self) -> None:
        """ Add a welcome message that is shown to the
        user before they submitted the first request
        """

        qimage = QImage("assets/logo-625.png")
        pixmap = QPixmap(qimage)
        pixmap_label = QLabel()
        pixmap_label.setPixmap(pixmap)

        grid = QGridLayout()
        grid.setColumnMinimumWidth(10, 0)
        grid.setAlignment(Qt.AlignTop)
        # grid.addWidget(pixmap_label, 1, 1, 3, 3)

        grid.addWidget(QLabel(""), 0, 0)
        grid.addWidget(QLabel(""), 0, 1)
        grid.addWidget(QLabel(""), 0, 2)
        grid.addWidget(QLabel(""), 1, 0)
        grid.addWidget(pixmap_label, 1, 1)
        grid.addWidget(QLabel(""), 1, 2)
        grid.addWidget(QLabel(""), 2, 0)
        grid.addWidget(
            QLabel("Thank you for using the W24 Services.\n "
                   "STEP 1: Use the checkboxes on the left to decide "
                   "which asks you want to submit to the API.\n"
                   "STEP 2: Pick the file you want to analyse "),
            2,
            1)
        grid.addWidget(QLabel(""), 2, 2)

        frame = QFrame()
        frame.setLayout(grid)
        self.addWidget(frame)

    def clear(self) -> None:
        """ Remove all widgets in the event feed
        """
        for i in reversed(range(self.count())):
            self.itemAt(i).widget().setParent(None)

    def add_json(
            self,
            json: str
    ) -> None:
        """ Add a JSON string to the feed

        Args:
            json (str): Json as string
        """
        json_editor = W24GuiJsonEditor()
        json_editor.setText(json)
        self.addWidget(json_editor)

    def add_headline(
            self,
            headline: str
    ) -> None:
        """ ADd a headline to the feed

        Args:
            headline (str): Headline as string
        """
        lb_headline = QLabel(headline)
        lb_headline.setFont(ft_headline)
        self.addWidget(lb_headline)

    def add_image(
            self,
            payload_bytes: Optional[bytes]
    ) -> None:
        """ Add an image to the feed

        Args:
            payload_bytes (Optional[bytes]): image in bytes
        """

        # if the payload is None, thre is no point
        if not payload_bytes:
            return

        # otherwise load the image
        qimage = QImage()
        qimage.loadFromData(payload_bytes)

        # rescale it
        pixmap = QPixmap(qimage)
        # pixmap = pixmap.scaled(1024, 1024, Qt.KeepAspectRatio)

        # turn it into a label
        pixmap_label = QLabel()
        pixmap_label.setPixmap(pixmap)

        # and add
        self.addWidget(pixmap_label)



import re
from os import path

from setuptools import setup

VERSIONFILE = "werk24/_version.py"
NAME = "werk24"


def _get_version(version_file: str)->str:
    version_pattern = r"^__version__ = ['\"]([^'\"]*)['\"]"
    with open(version_file, "rt") as file_handle:
        match = re.search(version_pattern, file_handle.read(), re.M)
    if match:
        return match.group(1)

    raise RuntimeError(
        "Unable to find version string in %s." %
        (VERSIONFILE,))


def _get_description() -> str:
    this_directory = path.abspath(path.dirname(__file__))
    with open(path.join(this_directory, 'README.md')) as file_handle:
        return file_handle.read()


setup(
    name=NAME,
    version=_get_version(VERSIONFILE),
    author="W24 Service GmbH",
    author_email="info@werk24.biz",
    description="Werk24 Client to read PDF- and Image-based "
    "Technical Drawings / Engineering Drawings",
    url="https://www.werk24.biz",
    entry_points={
        "console_scripts": [
            "w24cli=werk24.cli.w24cli:main",
            "w24gui=werk24.gui.w24gui:main",
        ]
    },
    extras_require={
        "gui": ["PyQt5", "pillow", "qscintilla"]
    },
    license='Business Source License 1.1',
    packages=[
        "werk24",
        "werk24.cli",
        "werk24.gui",
        "werk24.models",
    ],
    package_data={"werk24": ["assets/*.png"]},
    install_requires=[
        "aioboto3 >= 6.4.1",
        "pydantic >= 1.4",
        "python-dotenv>=0.10.1",
        "websockets >= 8.1"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Image Recognition"],
    python_requires='>=3.7.1',
    project_urls={
        "Documentation": "https://werk24.github.io/docs/"
    },
    long_description_content_type='text/markdown',
    long_description=_get_description(),
)




import argparse
import mock
import aiounittest
from werk24.cli.techread import main
from werk24.models.ask import W24AskType
from dotenv import load_dotenv

load_dotenv(".werk24")


class TechreadTest(aiounittest.AsyncTestCase):
    """ Small Test Case ensuring that the CLI associating
    interface does not change.
    """

    def _make_args(self):
        """ Make the args, requesting all asks
        """
        # mock the arguments
        args = argparse.Namespace()
        args.input_file = "."
        args.ignore_architecture_status = True
        args.ask_techread_started = True
        args.ask_page_thumbnail = True
        args.ask_sheet_thumbnail = True
        args.ask_sectional_thumbnail = True
        args.ask_variant_overall_dimensions = True
        args.ask_train = True
        return args

    @mock.patch('werk24.techread_client.W24TechreadClient')
    @mock.patch('werk24.cli.techread._get_drawing')
    async def test_ask_techread_started(
            self,
            get_drawing_mock,
            ClientMock):
        """ Test whether the cli-arguments translate into the
        correct W24AskTypes on the Hooks.
        """

        # mock the get_drawing function
        get_drawing_mock.return_value = ""

        # mock the args
        args = self._make_args()

        # call cli module
        await main(args)

        # get the list of Hooks
        client = ClientMock()
        async with client as session:
            hooks = session.read_drawing_with_hooks.call_args[0][1]
            ask_types = [h.ask.ask_type for h in hooks if h.ask is not None]

        # and compare
        assert W24AskType.PAGE_THUMBNAIL in ask_types
        assert W24AskType.SHEET_THUMBNAIL in ask_types
        assert W24AskType.SECTIONAL_THUMBNAIL in ask_types
        assert W24AskType.TRAIN in ask_types
